#include <stdio.h>
#include <string>
#include <iostream>
#include <sstream>
#include <map>
#include <vector>

using namespace std;

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////


typedef vector < int > Monomial; // sorted!

extern ostream & operator << ( ostream & os, const Monomial & m )
{
  os << "$";
  if ( m.size () == 0 )
    os << "1";

  for ( Monomial::const_iterator it = m.begin (); it != m.end (); ++ it )
    if ( * it > 0 )
      cout << "x_{" << * it << "}";
    else
      cout << "(1-x_{" << - * it << "})";

  os << "$";
  return os;
} 


typedef vector < Monomial > Polynomial;

typedef vector < Polynomial > GatesPolynomials;

bool AreEqual ( Monomial const & m1, Monomial const & m2 )
{
  if ( m1.size () != m2.size () )
    return false;

  for ( unsigned i = 0; i != m1.size (); ++i )
    if ( m1 [ i ] != m2 [ i ] )
      return false;

  return true;
}

Monomial MultiplyMonomials ( Monomial const & m1, Monomial const & m2 )
{
  Monomial m;
  size_t i1 = 0, i2 = 0;
  while ( i1 != m1.size () && i2 != m2.size () )
  {
    if ( m1 [ i1 ] < m2 [ i2 ] )
    {
      m.push_back ( m1 [ i1 ] );
      ++ i1;
    }
    else if ( m1 [ i1 ] > m2 [ i2 ] )
    {
      m.push_back ( m2 [ i2 ] );
      ++ i2;
    }
    else // m1 [ i1 ] == m2 [ i2 ]
    {
      m.push_back ( m1 [ i1 ] );
      ++ i1;
			++ i2;      
    }
  }

  while ( i1 != m1.size () )
  {
		m.push_back ( m1 [ i1 ] );
    ++ i1;
  }

  while ( i2 != m2.size () )
  {
		m.push_back ( m2 [ i2 ] );
    ++ i2;
  }

  return m;
}

int GetDegree ( Polynomial const & p )
{
  int deg = 0;

  for ( Polynomial::const_iterator it = p.begin (); it != p.end (); ++it )
    if ( it -> size () > deg )
      deg = it -> size ();

  return deg;
}

void AddMonomial ( Polynomial & P, Monomial const & m )
{
  for ( Polynomial::iterator it = P.begin (); it != P.end(); ++it )
    if ( AreEqual ( *it, m ) )
    {
      P.erase ( it );
      return;
    }

  P.push_back ( m );
}

void AddLiteralMonomial ( Polynomial & P, Monomial const & m )
{
  for ( unsigned i = 0; i != m.size (); ++i )
    if ( m [ i ] < 0 )
    {
      Monomial m1 = m;
      m1 [ i ] *= -1;

      Monomial m2;
      for ( Monomial::const_iterator t = m.begin (); t != m.end (); ++t )
        if ( * t != m [ i ] )
          m2.push_back ( * t );

      AddLiteralMonomial ( P, m1 );
      AddLiteralMonomial ( P, m2 );

      return;
    }

  AddMonomial ( P, m );
}

Monomial CloneMonomial ( Monomial const & m )
{
  Monomial newm;
  for ( Monomial::const_iterator it = m.begin (); it != m.end (); ++ it )
    newm.push_back ( * it );
  return newm;  
}

void Negate ( Polynomial & P )
{
  Monomial m;
  AddLiteralMonomial ( P, m );
}

void AddPolynomial ( Polynomial & P, Polynomial const & P2 )
{
  for ( Polynomial::const_iterator t = P2.begin (); t != P2.end (); ++t )
    AddLiteralMonomial ( P, CloneMonomial ( * t ) );
}

Polynomial ClonePolynomial ( Polynomial const & p )
{
  Polynomial newp;
  for ( Polynomial::const_iterator it = p.begin (); it != p.end (); ++ it )
    newp.push_back ( CloneMonomial ( * it) );
  return newp;  
}

Polynomial MultiplyPolynomials ( Polynomial const & p1, Polynomial const & p2 )
{
  Polynomial p;
	for ( Polynomial::const_iterator it1 = p1.begin (); it1 != p1.end (); ++ it1 )
    for ( Polynomial::const_iterator it2 = p2.begin (); it2 != p2.end (); ++ it2 )
      AddLiteralMonomial ( p, MultiplyMonomials ( * it1, * it2 ) );
  return p;
}

extern ostream & operator << ( ostream & os, const Polynomial & p )
{
  
  for ( Polynomial::const_iterator it = p.begin (); it != p.end (); ++ it )
  {
    os << * it;
    Polynomial::const_iterator tmp = it;
    ++tmp;
    if ( tmp != p.end () )    
      cout << " + ";
  }

  return os;
} 

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////


#define MAXOUTPUTS (10)
int inputsnum = 3, outputsnum = 1, gatesnum = 4;

pair<int, int> get_gate_coordinates ( int num )
{
#define XY(VAL,X,Y) case VAL: return make_pair(X,Y);
  switch ( gatesnum )
  {
    case 2:
      switch ( num ) {
        XY(0,2,11) XY(1,8,11) }
    case 3:
      switch ( num ) {
        XY(0,2,11) XY(1,8,11) XY(2,2,8) }
    case 4:
      switch ( num ) {
        XY(0,3,15) XY(1,6,14) XY(2,4,12) XY(3,5,11) }
    case 5:
      switch ( num ) {
        XY(0,3,15) XY(1,7,15) XY(2,2,10) XY(3,8,10) XY(4,5,7) }
    case 10:
      switch ( num ) {
        XY(0,2,14) XY(1,5,13) XY(2,8,13) XY(3,11,12) XY(4,2,11) 
        XY(5,6,10) XY(6,8,8) XY(7,4,7) XY(8,2,5) XY(9,4,5) }
    case 11:
      switch ( num ) {
        XY(0,2,14) XY(1,5,13) XY(2,8,13) XY(3,11,12) XY(4,2,11) 
        XY(5,6,10) XY(6,8,8) XY(7,4,7) XY(8,2,5) XY(9,4,5)
        XY(10,3,4) }
    case 12:
      switch ( num ) {
        XY(0,2,14) XY(1,5,13) XY(2,8,13) XY(3,11,12) XY(4,2,11) 
        XY(5,6,10) XY(6,8,8) XY(7,4,7) XY(8,2,5) XY(9,4,5)
        XY(10,3,4) }
    case 13:
      switch ( num ) {
        XY(0,2,14) XY(1,5,13) XY(2,8,13) XY(3,11,12) XY(4,2,11) 
        XY(5,6,10) XY(6,8,8) XY(7,4,7) XY(8,2,5) XY(9,4,5)
        XY(10,3,4) XY(11,7,3) XY(12,10,2) }
    case 14:
      switch ( num ) {
        XY(0,2,14) XY(1,5,13) XY(2,8,13) XY(3,11,12) XY(4,2,11) 
        XY(5,6,10) XY(6,8,8) XY(7,4,7) XY(8,2,5) XY(9,4,5)
        XY(10,3,4) XY(11,7,3) XY(12,10,2) XY(13,5,1)}
    case 15:
      switch ( num ) {
        XY(0,2,14) XY(1,5,13) XY(2,8,13) XY(3,11,12) XY(4,2,11) 
        XY(5,6,10) XY(6,8,8) XY(7,4,7) XY(8,2,5) XY(9,4,5)
        XY(10,3,4) XY(11,7,3) XY(12,10,2) XY(13,5,1) XY(14,8,0) }

    case 6:
      switch ( num )
      {
        case 0: return make_pair ( 2, 10 + 4 );
        case 1: return make_pair ( 5, 9 + 4 );
        case 2: return make_pair ( 8, 9 + 4 );
        case 3: return make_pair ( 11, 8 + 4 );
        case 4: return make_pair ( 2, 7 + 4 );
        case 5: return make_pair ( 6, 6 + 4 );
      }
    case 7:
      switch ( num )
      {
        case 0: return make_pair ( 2, 10 + 4 );
        case 1: return make_pair ( 5, 9 + 4  );
        case 2: return make_pair ( 8, 9 + 4  );
        case 3: return make_pair ( 11, 8 + 4  );
        case 4: return make_pair ( 2, 7 + 4  );
        case 5: return make_pair ( 6, 6 + 4  );
        case 6: return make_pair ( 8, 4 + 4  );
      }
    case 8:
      switch ( num )
      {
        case 0: return make_pair ( 2, 10 + 4  );
        case 1: return make_pair ( 5, 9 + 4  );
        case 2: return make_pair ( 8, 9 + 4  );
        case 3: return make_pair ( 11, 8 + 4  );
        case 4: return make_pair ( 2, 7 + 4  );
        case 5: return make_pair ( 6, 6 + 4  );
        case 6: return make_pair ( 8, 4 + 4  );
        case 7: return make_pair ( 4, 3 + 4  );  
      }
    case 9:
      switch ( num )
      {
        case 0: return make_pair ( 2, 10 + 4 );
        case 1: return make_pair ( 5, 9 + 4 );
        case 2: return make_pair ( 8, 9 + 4 );
        case 3: return make_pair ( 11, 8 + 4 );
        case 4: return make_pair ( 2, 7 + 4 );
        case 5: return make_pair ( 6, 6 + 4 );
        case 6: return make_pair ( 8, 4 + 4 );
        case 7: return make_pair ( 4, 3 + 4 );  
        case 8: return make_pair ( 2, 1 + 4 );  
      }

    default:
      return make_pair ( 3, 3 );
  }

  return make_pair ( 3, 3 );
}

int outputs[MAXOUTPUTS];


int main() {
  int i, j, k;

  freopen("scheme.out", "r", stdin);
  
  scanf("%d", &inputsnum);
  scanf("%d", &gatesnum);
  scanf("%d", &outputsnum);

  GatesPolynomials GP;

  for ( j = 0; j != outputsnum; ++j )
    scanf("%d", &(outputs[j]));

  cout << "\n    \\myfigure{";
  cout << "\n      \\draw[very thin,color=white!85!red,step=1.0] (0,0) grid (15,17);";
  cout << "\n      \\foreach \\x in {0,...,15} { \\draw(-0.5,\\x) node{\\x}; \\draw(\\x,-0.5) node{\\x};  \\draw(\\x, 17.5) node{\\x}; }";

  for ( i = 0; i != inputsnum; ++i )
  {
    cout << "\n      \\mysimpleinput{x" << i << "}{(" << 2*i+2 << ",17)}{$x_{" << i+1 << "}$}";
    Monomial m;
    m.push_back ( i + 1 );
    Polynomial p;
    p.push_back ( m );
    GP.push_back ( p );
  }

  
    


  for ( k = 0; k != gatesnum; ++k )
  {
    int in1, in2;
    scanf("%d: ", &in1);
    scanf("(%d, ", &in1);
    scanf("%d, ", &in2);
    in1 -= 1;
    in2 -= 1;

    Polynomial p;
    Polynomial p1 = ClonePolynomial ( GP [ in1 ] );
    Polynomial p2 = ClonePolynomial ( GP [ in2 ] );

    int type;
    scanf("%d)\n", &type);

    int x = get_gate_coordinates ( k ).first;
    int y = get_gate_coordinates ( k ).second;

    string op = "", leftlab = "", rightlab = "";
    bool leftlabneg = false, rightlabneg = false;

    switch ( type )
    {
      case  0: op = "?!"; break;
      case  1: {
                 op = "\\land"; leftlab = "$\\neg$"; rightlab = "$\\neg$"; leftlabneg = true; rightlabneg = true;
                 Negate ( p1 );
                 Negate ( p2 );
								 p = MultiplyPolynomials ( p1, p2 );
                 break;
               }
      case  2: {
						     op = "\\land"; leftlab = "$\\neg$"; leftlabneg = true;
                 Negate ( p1 );
								 p = MultiplyPolynomials ( p1, p2 );
								 break;
               }
      case  3: op = "?!"; break;
      case  4: {
                 op = "\\land"; rightlab = "$\\neg$"; rightlabneg = true;
                 Negate ( p2 );
								 p = MultiplyPolynomials ( p1, p2 );
								 break;
               }
      case  5: op = "?!"; break;
      case  6: {
                 op = "\\oplus"; 
								 AddPolynomial ( p, p1 );
								 AddPolynomial ( p, p2 );
                 break;
               }
      case  7: {
								 op = "\\lor"; leftlab = "$\\neg$"; rightlab = "$\\neg$"; leftlabneg = true; rightlabneg = true;
                 p = MultiplyPolynomials ( p1, p2 );
                 Negate ( p );
                 break;
               }
      case  8: { 
                 op = "\\land"; 
								 p = MultiplyPolynomials ( p1, p2 );
                 break;
               }
      case  9: {
                 op = "\\equiv"; 
								 AddPolynomial ( p, p1 );
								 AddPolynomial ( p, p2 );
                 Negate ( p );
                 break;
               }
      case 10: op = "?!"; break;
      case 11: {
							   op = "\\lor"; leftlab = "$\\neg$"; leftlabneg = true; 
                 AddPolynomial ( p, p1 );
                 AddPolynomial ( p, MultiplyPolynomials ( p1, p2 ) );
								 Negate ( p );
                 break;
               }
      case 12: op = "?!"; break;
      case 13: {
                 op = "\\lor"; rightlab = "$\\neg$"; rightlabneg = true;
                 AddPolynomial ( p, p2 );
                 AddPolynomial ( p, MultiplyPolynomials ( p1, p2 ) );
								 Negate ( p );
                 break;
               }
      case 14: {
                 op = "\\lor"; 
                 AddPolynomial ( p, p1 );
                 AddPolynomial ( p, p2 );
                 AddPolynomial ( p, MultiplyPolynomials ( p1, p2 ) );
                 break;
               }
      case 15: op = "?!"; break;
      
      default: op = "?!"; 
    }

    GP.push_back ( p );


    cout << "\n      \\mylabeledgatetwo";
			   if ( ! rightlabneg && ! leftlabneg ) cout << "pospos{x";
    else if (   rightlabneg && ! leftlabneg ) cout << "negpos{x";
    else if ( ! rightlabneg &&   leftlabneg ) cout << "posneg{x";
    else if (   rightlabneg &&   leftlabneg ) cout << "negneg{x";


    cout << inputsnum + k << "}{(" << x << "," << y << ")}{$" << 
      op << "$}{$g_{" << inputsnum + k + 1 << "}$}{x" << in1 << "}{x" << in2 << "}";
    

    //cout << "\n      \\mylabeledgatezero{x" << inputsnum + k << "}{(" << x << "," << y << ")}{$" << 
    //   op << "$}{$g_{" << inputsnum + k + 1 << "}$}"; //{x" << in1 << "}{x" << in2 << "}";
    //cout << "\n      \\mylabelededge{x" << in1 << "}{x" << inputsnum + k << "}{" << leftlab  << "}";
    //cout << "\n      \\mylabelededge{x" << in2 << "}{x" << inputsnum + k << "}{" << rightlab << "}";
  } 

  for ( k = 0; k != outputsnum; ++k )
    cout << "\n      \\myemptygateone{z" << k << "}{(" << 3*k+2 << "," << "-2)}{$z_" << k << "$}{x" << outputs[k]-1 << "}";

  cout << "\n    }{}";

  fclose(stdin);

  cout << "\n\nPolynomials: \\begin{itemize}";
  //for ( GatesPolynomials::const_iterator it = GP.begin (); it != GP.end (); ++it )
    //cout << "\n \\item " << * it;
  for ( size_t y = 0; y != GP.size (); ++y )
    cout << "\n \\item $g_{" << y + 1 << "}$: " << GP [ y ];
  cout << "\n\\end{itemize}";

  
  

  return 0;
}
